@@ -42,9 +42,11 @@
     post.psi     <- post.mu
     ll.store     <- vector("integer", n.store)
 
-    mu.sigma     <- 1/sigma.mu
-    mu.zero      <- as.numeric(mu.zero)
-    mu.prior     <- mu.sigma * as.numeric(mu.zero)
+    if(update.mu <- sw["u.sw"]) {
+      mu.sigma   <- 1/sigma.mu
+      mu.zero    <- as.numeric(mu.zero)
+      mu.prior   <- mu.sigma * as.numeric(mu.zero)
+    } else mu[]  <- 0L
     uni.type     <- switch(EXPR=uni.type,  unconstrained=,               constrained="constrained", "single")
     .sim_psi_inv <- switch(EXPR=uni.type,  constrained=.sim_psi_u1,      single=.sim_psi_c1)
     .sim_psi_ip  <- switch(EXPR=uni.prior, unconstrained=.sim_psi_ipu,   isotropic=.sim_psi_ipc)
@@ -89,13 +91,15 @@
       psi.inv[]  <- .sim_psi_inv(uni.shape, psi.beta, S.mat, V)
 
     # Means
-      mu[]       <- .sim_mu(N=N, P=P, mu.sigma=mu.sigma, psi.inv=psi.inv, sum.data=sum.data, sum.eta=colSums2(eta), lmat=lmat, mu.prior=mu.prior)
+      if(update.mu) {
+        mu[]     <- .sim_mu(N=N, P=P, mu.sigma=mu.sigma, psi.inv=psi.inv, sum.data=sum.data, sum.eta=colSums2(eta), lmat=lmat, mu.prior=mu.prior)
+      }
 
       if(storage) {
         if(verbose) utils::setTxtProgressBar(pb, iter)
         new.it   <- which(iters == iter)
         psi      <- 1/psi.inv
-        post.mu  <- post.mu + mu/n.store
+        post.mu  <- post.mu  + mu/n.store
         post.psi <- post.psi + psi/n.store
         if(sw["mu.sw"])             mu.store[,new.it]   <- mu
         if(all(sw["s.sw"], Q0))   eta.store[,,new.it]   <- eta
