@@ -42,9 +42,11 @@
     nu1.5        <- nu1 + 0.5
     P.5          <- P/2
 
-    mu.sigma     <- 1/sigma.mu
-    mu.zero      <- as.numeric(mu.zero)
-    mu.prior     <- mu.sigma * mu.zero
+    if(update.mu <- sw["u.sw"]) {
+      mu.sigma   <- 1/sigma.mu
+      mu.zero    <- as.numeric(mu.zero)
+      mu.prior   <- mu.sigma * mu.zero
+    } else mu[]  <- 0L
     uni.type     <- switch(EXPR=uni.type,  unconstrained=,               constrained="constrained", "single")
     .sim_psi_inv <- switch(EXPR=uni.type,  constrained=.sim_psi_u1,      single=.sim_psi_c1)
     .sim_psi_ip  <- switch(EXPR=uni.prior, unconstrained=.sim_psi_ipu,   isotropic=.sim_psi_ipc)
@@ -122,7 +124,9 @@
       psi.inv[]  <- .sim_psi_inv(uni.shape, psi.beta, S.mat, V)
 
     # Means
-      mu[]       <- .sim_mu(N=N, P=P, mu.sigma=mu.sigma, psi.inv=psi.inv, sum.data=sum.data, sum.eta=colSums2(eta), lmat=lmat, mu.prior=mu.prior)
+      if(update.mu) {
+        mu[]     <- .sim_mu(N=N, P=P, mu.sigma=mu.sigma, psi.inv=psi.inv, sum.data=sum.data, sum.eta=colSums2(eta), lmat=lmat, mu.prior=mu.prior)
+      }
 
     # Shrinkage
       if(Q0) {
